
using System;
using System.Collections.Generic;
using System.Text;
using ScrewTurn.Wiki.PluginFramework;
using System.Web;

namespace ScrewTurn.Wiki {

    /// <summary>
    /// Allows access to the Pages.
    /// </summary>
    public class Pages {

		private static Pages instance = null;

		/// <summary>
		/// Gets or sets the singleton instance of the Pages object.
		/// </summary>
		public static Pages Instance {
			get { return instance; }
			set { instance = value; }
		}

		private List<PageInfo> pages = null; // Sorted
		private List<CategoryInfo> categories = null; // Sorted

        /// <summary>
        /// Initializes a new instance of the <b>Pages</b> class.
        /// </summary>
        public Pages() {
            pages = new List<PageInfo>();
			categories = new List<CategoryInfo>();
            Load();
        }

		/// <summary>
		/// Loads Pages and Categories from all the Providers.
		/// </summary>
        public void Load() {
            lock(pages) {
				IPagesStorageProvider[] provs = Collectors.PagesProviderCollector.AllProviders;
				for(int i = 0; i < provs.Length; i++) {
					LoadFrom(provs[i], false);
				}
                
				// Sort is very important for binary search
                pages.Sort(new PageNameComparer());
				categories.Sort(new CategoryNameComparer());
            }
        }

		/// <summary>
		/// Loads Pages and Categories from a specific Provider.
		/// </summary>
		/// <param name="provider">The Provider.</param>
		/// <param name="autosort">If true, the global pages list is sorted after the operation.</param>
		public void LoadFrom(IPagesStorageProvider provider, bool autosort) {
			lock(pages) lock(categories) {
				pages.AddRange(provider.AllPages);
				categories.AddRange(provider.AllCategories);
				if(autosort) {
					pages.Sort(new PageNameComparer());
					categories.Sort(new CategoryNameComparer());
				}
			}
		}

		/// <summary>
		/// Unloads all the data provided by the specified Provider.
		/// </summary>
		/// <param name="provider">The Provider.</param>
		public void UnloadFrom(IPagesStorageProvider provider) {
			lock(pages) {
				lock(categories) {
					List<PageInfo> pagesToRemove = new List<PageInfo>();
					List<CategoryInfo> catsToRemove = new List<CategoryInfo>();
					for(int i = 0; i < pages.Count; i++) {
						if(pages[i].Provider == provider) pagesToRemove.Add(pages[i]);
					}
					for(int i = 0; i < pagesToRemove.Count; i++) {
						pages.Remove(pagesToRemove[i]);
					}
					for(int i = 0; i < categories.Count; i++) {
						if(categories[i].Provider == provider) catsToRemove.Add(categories[i]);
					}
					for(int i = 0; i < catsToRemove.Count; i++) {
						categories.Remove(catsToRemove[i]);
					}
					Content.InvalidateAll();
					Content.PseudoCache.Clear();
				}
			}
		}

		/// <summary>
		/// Reloads all the Pages and Categories managed by the specified provider.
		/// </summary>
		/// <param name="provider">The Provider.</param>
		public void ReloadFrom(IPagesStorageProvider provider) {
			lock(pages) {
				lock(categories) {
					List<PageInfo> pagesToRemove = new List<PageInfo>();
					List<CategoryInfo> catsToRemove = new List<CategoryInfo>();
					for(int i = 0; i < pages.Count; i++) {
						if(pages[i].Provider == provider) pagesToRemove.Add(pages[i]);
					}
					for(int i = 0; i < pagesToRemove.Count; i++) {
						pages.Remove(pagesToRemove[i]);
					}
					for(int i = 0; i < categories.Count; i++) {
						if(categories[i].Provider == provider) catsToRemove.Add(categories[i]);
					}
					for(int i = 0; i < catsToRemove.Count; i++) {
						categories.Remove(catsToRemove[i]);
					}
					LoadFrom(provider, true);
					Content.InvalidateAll();
					Content.PseudoCache.Clear();
				}
			}
		}

		/// <summary>
		/// Reloads the Pages cache.
		/// </summary>
		public void Reload() {
			lock(pages) {
				pages.Clear();
			}
			lock(categories) {
				categories.Clear();
			}
			Load();
		}

		#region Pages

		/// <summary>
        /// Finds a Page.
        /// </summary>
        /// <param name="name">The Name of the Page to find (case <b>unsensitive</b>).</param>
        /// <returns>The Page Info, if any, null otherwise.</returns>
        public PageInfo FindPage(string name) {
			if(name == null || name.Length == 0) return null;
            lock(pages) {
				int idx = pages.BinarySearch(new PageInfo(name, null, PageStatus.Normal, DateTime.Now), new PageNameComparer());
				if(idx >= 0) return pages[idx];
            }
            return null;
        }

		/// <summary>
		/// Finds a Page in a specific Provider.
		/// </summary>
		/// <param name="name">The Name of the Page to find (case <b>unsensitive</b>).</param>
		/// <param name="provider">The Provider.</param>
		/// <returns>The Page Info, if any, null otherwise.</returns>
		public PageInfo FindPage(string name, IPagesStorageProvider provider) {
			if(name == null || name.Length == 0) return null;
			lock(pages) {
				int idx = pages.BinarySearch(new PageInfo(name, null, PageStatus.Normal, DateTime.Now), new PageNameComparer());
				if(idx >= 0 && pages[idx].Provider == provider) return pages[idx];
			}
			return null;
		}

		/// <summary>
		/// Gets the Backups/Revisions of a Page.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <returns>The list of available Backups/Revision numbers.</returns>
		public List<int> GetBackups(PageInfo page) {
			return page.Provider.GetBackups(page);
		}

		/// <summary>
		/// Gets the Content of a Page Backup.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="revision">The Backup/Revision number.</param>
		/// <returns>The Content of the Backup.</returns>
		public PageContent GetBackupContent(PageInfo page, int revision) {
			return page.Provider.GetBackupContent(page, revision);
		}

		private void Backup(PageInfo page) {
			int kept = Settings.KeptBackupNumber;

			if(kept != 0) { // This avoids creating a backup that would be deleted
				bool done = page.Provider.Backup(page);
				if(done) Log.LogEntry("Backup created for " + page.Name, EntryType.General, "SYSTEM");
				else {
					Log.LogEntry("Backup failed for " + page.Name, EntryType.Error, "SYSTEM");
					return;
				}
			}

			List<int> backups = page.Provider.GetBackups(page);
			if(kept >= 0) {
				if(backups.Count > kept) {
					page.Provider.DeleteBackups(page, backups.Count - kept - 1);
				}
			}
		}

		/// <summary>
		/// Deletes all the Page Backups.
		/// </summary>
		/// <param name="page">The Page.</param>
		public bool DeleteBackups(PageInfo page) {
			return page.Provider.DeleteBackups(page, -1);
		}

		/// <summary>
		/// Deletes a part of the Page Backups.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="firstToDelete">The first backup to be deleted (this backup and older backups are deleted).</param>
		public bool DeleteBackups(PageInfo page, int firstToDelete) {
			bool done = page.Provider.DeleteBackups(page, firstToDelete);
			if(done) {
				Log.LogEntry("Backups (0-" + firstToDelete.ToString() + ") deleted for " + page.Name, EntryType.General, "SYSTEM");
			}
			else {
				Log.LogEntry("Backups (0-" + firstToDelete.ToString() + ") deletion failed for " + page.Name, EntryType.Error, "SYSTEM");
			}
			return done;
		}

		/// <summary>
		/// Performs the rollpack of a Page.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="version">The revision to rollback the Page to.</param>
		public bool Rollback(PageInfo page, int version) {
			bool done = page.Provider.RollbackPage(page, version);

			if(done) {
				Content.Invalidate(page);
				Log.LogEntry("Rollback executed for " + page.Name + " at revision " + version.ToString(), EntryType.General, "SYSTEM");
				RecentChanges.Instance.AddChange(page.Name, Content.GetPageContent(page, false).Title, DateTime.Now, SessionFacade.Username, Change.PageRolledBack, "");
				return true;
			}
			else {
				Log.LogEntry("Rollback failed for " + page.Name + " at revision " + version.ToString(), EntryType.Error, "SYSTEM");
				return false;
			}
        }

		/// <summary>
		/// Creates a new Page.
		/// </summary>
		/// <param name="name">The Page name.</param>
		/// <returns>True if the Page has been created successfully.</returns>
        public bool CreatePage(string name) {
			return CreatePage(name, Collectors.PagesProviderCollector.GetProvider(Settings.DefaultPagesProvider));
        }

		/// <summary>
		/// Creates a new Page.
		/// </summary>
		/// <param name="name">The Page name.</param>
		/// <returns>True if the Page has been created successfully.</returns>
		public bool CreatePage(string name, IPagesStorageProvider provider) {
			if(FindPage(name) != null) return false;

			PageInfo newPage = provider.AddPage(name, DateTime.Now);

			if(newPage != null) {
				lock(pages) {
					pages.Add(newPage);
					pages.Sort(new PageNameComparer());
				}
				Content.InvalidateAll();
				Content.PseudoCache.Clear();
				Log.LogEntry("Page " + name + " created", EntryType.General, "SYSTEM");
				return true;
			}
			else {
				Log.LogEntry("Page creation failed for " + name, EntryType.Error, "SYSTEM");
				return false;
			}
		}

		/// <summary>
		/// Sets the Status of a Page.
		/// </summary>
		/// <param name="page">The Page to set the status of.</param>
		/// <param name="status">The Status.</param>
		/// <returns>True if the Status has been set successfully.</returns>
		public bool SetPageStatus(PageInfo page, PageStatus status) {
			PageInfo newPage = page.Provider.SetStatus(page, status);
			if(newPage != null) {
				lock(pages) {
					pages.Remove(page);
					pages.Add(newPage);
					pages.Sort(new PageNameComparer());
				}
				Log.LogEntry("Page Status changed to " + status.ToString() + " for " + page.Name, EntryType.General, "SYSTEM");
			}
			else Log.LogEntry("Page Status change failed for " + page.Name + " (" + status.ToString() + ")", EntryType.Error, "SYSTEM");
			return newPage != null;
		}

        /// <summary>
        /// Deletes a Page.
        /// </summary>
        /// <param name="page">The Page to delete.</param>
        public bool DeletePage(PageInfo page) {
			string title = Content.GetPageContent(page, false).Title;

			bool done = page.Provider.RemovePage(page);

			if(done) {
				lock(pages) {
					lock(categories) {
						ReloadFrom(page.Provider);
						pages.Sort(new PageNameComparer());
						categories.Sort(new CategoryNameComparer());
					}
				}
				// Remove the deleted page from the Breadcrumbs Trail and Redirections list
				SessionFacade.Breadcrumbs.RemovePage(page);
				Redirections.Instance.WipePageOut(page);
				// Cleanup Cache
				Content.Invalidate(page);
				Content.PseudoCache.Clear();
				Log.LogEntry("Page " + page.Name + " deleted", EntryType.General, "SYSTEM");
				RecentChanges.Instance.AddChange(page.Name, title, DateTime.Now, SessionFacade.Username, Change.PageDeleted, "");
				return true;
			}
			else {
				Log.LogEntry("Page deletion failed for " + page.Name, EntryType.Error, "SYSTEM");
				return false;
			}
        }

        /// <summary>
        /// Renames a Page.
        /// </summary>
        /// <param name="page">The Page to rename.</param>
        /// <param name="name">The new name.</param>
        public bool RenamePage(PageInfo page, string name) {
			if(FindPage(name) != null) return false;

			string title = Content.GetPageContent(page, false).Title;

			PageInfo pg = page.Provider.RenamePage(page, name);
			if(pg != null) {
				lock(pages) {
					lock(categories) {
						ReloadFrom(page.Provider);
						pages.Sort(new PageNameComparer());
						categories.Sort(new CategoryNameComparer());
					}
				}
				SessionFacade.Breadcrumbs.RemovePage(page);
				Redirections.Instance.Clear();
				Content.InvalidateAll();
				Content.PseudoCache.Clear();
				Log.LogEntry("Page " + page.Name + " renamed to " + name, EntryType.General, "SYSTEM");
				RecentChanges.Instance.AddChange(page.Name, title, DateTime.Now, SessionFacade.Username, Change.PageRenamed, "");
				return true;
			}
			else {
				Log.LogEntry("Page rename failed for " + page.Name + " (" + name + ")", EntryType.Error, "SYSTEM");
				return false;
			}
        }

		/// <summary>
		/// Modifies a Page.
		/// </summary>
		/// <param name="page">The Page to modify.</param>
		/// <param name="title">The Title of the Page.</param>
		/// <param name="username">The Username of the user who modified the Page.</param>
		/// <param name="dateTime">The Date/Time of the modification.</param>
		/// <param name="comment">The comment of the editor, about this revision.</param>
		/// <param name="content">The Content.</param>
		/// <param name="backup">A value specifying whether or not backup the Page.</param>
		/// <returns>True if the Page has been modified successfully.</returns>
		public bool ModifyPage(PageInfo page, string title, string username, DateTime dateTime, string comment, string content, bool backup) {
			StringBuilder sb = new StringBuilder(content);
			sb.Replace("~~~~", "(" + username + "," + dateTime.ToString("yyyy'/'MM'/'dd' 'HH':'mm':'ss") + ")");
			content = sb.ToString();

			bool done = page.Provider.ModifyPage(page, title, username, dateTime, comment, content, backup);
			if(done) {
				// Because of transclusion, it is necessary to clear the whole cache
				//Content.Invalidate(page);
				Content.InvalidateAll();
				Log.LogEntry("Page Content updated for " + page.Name, EntryType.General, "SYSTEM");
				RecentChanges.Instance.AddChange(page.Name, title, dateTime, username, Change.PageUpdated, comment);
			}
			else Log.LogEntry("Page Content update failed for " + page.Name, EntryType.Error, "SYSTEM");
			return done;
		}

        /// <summary>
        /// Gets the list of all the Pages of the Wiki. The List shouldn't be modified.
        /// </summary>
        public List<PageInfo> AllPages {
			get { return pages; }
        }

		/// <summary>
		/// Finds a Category.
		/// </summary>
		/// <param name="name">The name of the Category to Find (case <b>unsensitive</b>).</param>
		/// <returns>The CategoryInfo or null.</returns>
		public CategoryInfo FindCategory(string name) {
			if(name == null || name.Length == 0) return null;
			lock(categories) {
				int idx = categories.BinarySearch(new CategoryInfo(name, null), new CategoryNameComparer());
				if(idx >= 0) return categories[idx];
			}
			return null;
		}

		#endregion

		#region Categories

		/// <summary>
		/// Creates a new Category.
		/// </summary>
		/// <param name="name">The Name of the Category.</param>
		/// <returns>True if the Category has been created successfully.</returns>
		public bool CreateCategory(string name) {
			return CreateCategory(name, Collectors.PagesProviderCollector.GetProvider(Settings.DefaultPagesProvider));
		}

		/// <summary>
		/// Creates a new Category in the specifued Provider.
		/// </summary>
		/// <param name="name">The Name of the Category.</param>
		/// <param name="provider">The Provider.</param>
		/// <returns>True if the Category has been created successfully.</returns>
		public bool CreateCategory(string name, IPagesStorageProvider provider) {
			if(FindCategory(name) != null) return false;

			if(provider == null) provider = Collectors.PagesProviderCollector.GetProvider(Settings.DefaultPagesProvider);

			CategoryInfo newCat = provider.AddCategory(name);

			if(newCat != null) {
				lock(categories) {
					categories.Add(newCat);
					categories.Sort(new CategoryNameComparer());
				}
				Log.LogEntry("Category " + name + " created", EntryType.General, "SYSTEM");
				return true;
			}
			else {
				Log.LogEntry("Category creation failed for " + name, EntryType.Error, "SYSTEM");
				return false;
			}
		}

		/// <summary>
		/// Removes a Category.
		/// </summary>
		/// <param name="category">The Category to remove.</param>
		/// <returns>True if the Category has been removed successfully.</returns>
		public bool RemoveCategory(CategoryInfo category) {
			bool done = category.Provider.RemoveCategory(category);
			if(done) {
				lock(categories) {
					categories.Remove(category);
				}
				Log.LogEntry("Category " + category.Name + " removed", EntryType.General, "SYSTEM");
			}
			else Log.LogEntry("Category deletion failed for " + category.Name, EntryType.Error, "SYSTEM");
			return done;
		}

		/// <summary>
		/// Renames a Category.
		/// </summary>
		/// <param name="category">The Category to rename.</param>
		/// <param name="newName">The new Name of the Category.</param>
		/// <returns>True if the Category has been renamed successfully.</returns>
		public bool RenameCategory(CategoryInfo category, string newName) {
			if(FindCategory(newName) != null) return false;

			string oldName = category.Name;

			CategoryInfo newCat = category.Provider.RenameCategory(category, newName);

			if(newCat != null) {
				lock(categories) {
					categories.Remove(category);
					categories.Add(newCat);
					categories.Sort(new CategoryNameComparer());
				}
				Log.LogEntry("Category " + oldName + " renamed to " + newName, EntryType.General, "SYSTEM");
			}
			else Log.LogEntry("Category rename failed for " + oldName + " (" + newName + ")", EntryType.Error, "SYSTEM");
			return newCat != null;
		}

		/// <summary>
		/// Gets the Categories of a Page.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <returns>The Categories of the Page.</returns>
		public CategoryInfo[] GetCategoriesPerPage(PageInfo page) {
			List<CategoryInfo> result = new List<CategoryInfo>();
			lock(categories) {
				for(int i = 0; i < categories.Count; i++) {
					if(ContainsString(categories[i].Pages, page.Name)) result.Add(categories[i]);
				}
			}
			return result.ToArray();
		}

		/// <summary>
		/// Gets all the Uncategorized Pages.
		/// </summary>
		/// <returns>The Uncategorized Pages.</returns>
		public PageInfo[] GetUncategorizedPages() {
			List<PageInfo> result = new List<PageInfo>();
			lock(pages) {
				lock(categories) {
					for(int i = 0; i < pages.Count; i++) {
						bool found = false;
						for(int k = 0; k < categories.Count; k++) {
							if(ContainsString(categories[k].Pages, pages[i].Name)) {
								found = true;
								break;
							}
						}
						if(!found) result.Add(pages[i]);
					}
				}
			}
			return result.ToArray();
		}

		/// <summary>
		/// Gets the valid Categories for a Page, i.e. the Categories managed by the Page's Provider.
		/// </summary>
		/// <param name="page">The Page, or null to use the default provider.</param>
		/// <returns>The valid Categories.</returns>
		public CategoryInfo[] GetAvailableCategories(PageInfo page) {
			if(page != null) {
				return page.Provider.AllCategories;
			}
			else {
				return Collectors.PagesProviderCollector.GetProvider(Settings.DefaultPagesProvider).AllCategories;
			}
		}

		/// <summary>
		/// Gets the other Categories of the Provider of the specified Category.
		/// </summary>
		/// <param name="category">The Category.</param>
		/// <returns>The matching Categories.</returns>
		public CategoryInfo[] GetMatchingCategories(CategoryInfo category) {
			List<CategoryInfo> result = new List<CategoryInfo>();
			lock(categories) {
				for(int i = 0; i < categories.Count; i++) {
					if(categories[i].Provider == category.Provider && categories[i] != category) {
						result.Add(categories[i]);
					}
				}
			}
			return result.ToArray();
		}

		private bool ContainsPageInfo(PageInfo[] pages, PageInfo page) {
			PageNameComparer comp = new PageNameComparer();
			for(int i = 0; i < pages.Length; i++) {
				if(comp.Compare(pages[i], page) == 0) return true;
			}
			return false;
		}

		/// <summary>
		/// Binds a Page with some Categories.
		/// </summary>
		/// <param name="page">The Page to rebind.</param>
		/// <param name="categories">The Categories to bind the Page with.</param>
		/// <remarks>
		/// The specified Categories must be managed by the same Provider that manages the Page.
		/// The operation removes all the previous bindings.
		/// </remarks>
		/// <returns>True if the binding succeeded.</returns>
		public bool Rebind(PageInfo page, CategoryInfo[] cats) {
			string[] names = new string[cats.Length];
			for(int i = 0; i < cats.Length; i++) {
				if(cats[i].Provider != page.Provider) return false;
				names[i] = cats[i].Name; // Saves one cycle
			}
			bool done = page.Provider.Rebind(page, names);
			if(done) {
				// Update data in memory (this implementation is not very efficient)
				lock(categories) {
					// 1 - Remove the Page from ALL the Categories
					for(int i = 0; i < categories.Count; i++) {
						int idx;
						if(ContainsString(categories[i].Pages, page.Name, out idx)) {
							List<string> tmp = new List<string>(categories[i].Pages);
							tmp.RemoveAt(idx);
							categories[i].Pages = tmp.ToArray();
						}
					}
					// 2 - Add the Page to the selected Categories
					for(int i = 0; i < cats.Length; i++) {
						List<string> tmp = new List<string>(cats[i].Pages);
						tmp.Add(page.Name);
						cats[i].Pages = tmp.ToArray();
					}
				}
				Log.LogEntry("Page " + page.Name + " rebound", EntryType.General, "SYSTEM");
			}
			else Log.LogEntry("Page rebind failed for " + page.Name, EntryType.Error, "SYSTEM");
			return done;
		}

		private bool ContainsString(string[] pages, string page) {
			int idx;
			return ContainsString(pages, page, out idx);
		}

		private bool ContainsString(string[] pages, string page, out int idx) {
			for(int i = 0; i < pages.Length; i++) {
				if(pages[i].ToLower().Equals(page.ToLower())) {
					idx = i;
					return true;
				}
			}
			idx = -1;
			return false;
		}

		/// <summary>
		/// Merges two Categories.
		/// </summary>
		/// <param name="source">The source Category.</param>
		/// <param name="destination">The destination Category.</param>
		/// <returns>True if the Categories have been merged successfully.</returns>
		/// <remarks>The <b>destination</b> Category remains, while the <b>source</b> Category is deleted, and all its Pages re-binded in the <b>destination</b> Category.
		/// The two Categories must have the same provider.</remarks>
		public bool MergeCategories(CategoryInfo source, CategoryInfo destination) {
			if(source.Provider != destination.Provider) return false;

			CategoryInfo newCat = source.Provider.MergeCategories(source, destination);

			if(newCat != null) {
				// Update data in memory
				lock(categories) {
					// 1 - Remove Source and Destination
					categories.Remove(source);
					categories.Remove(destination);
					// 2 - Add new Category
					categories.Add(newCat);
					categories.Sort(new CategoryNameComparer());
				}
				Log.LogEntry("Category " + source.Name + " merged into " + destination.Name, EntryType.General, "SYSTEM");
			}
			else Log.LogEntry("Categories merging failed for " + source.Name + " into " + destination.Name, EntryType.Error, "SYSTEM");
			return newCat != null;
		}

		/// <summary>
		/// Gets the list of all the Categories of the Wiki. The list shouldn't be modified.
		/// </summary>
		public List<CategoryInfo> AllCategories {
			get { return categories; }
		}

		#endregion

		#region Page Discussion

		/// <summary>
		/// Gets the Page Messages.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <returns>The list of the <b>first-level</b> Messages, containing the replies properly nested.</returns>
		public Message[] GetPageMessages(PageInfo page) {
			return page.Provider.GetMessages(page);
		}

		/// <summary>
		/// Gets the total number of Messages in a Page Discussion.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <returns>The number of messages.</returns>
		public int GetMessageCount(PageInfo page) {
			return page.Provider.GetMessageCount(page);
		}

		/// <summary>
		/// Finds a Message.
		/// </summary>
		/// <param name="messages">The Messages.</param>
		/// <param name="id">The Message ID.</param>
		/// <returns>The Message or null.</returns>
		public Message FindMessage(List<Message> messages, int id) {
			Message result = null;
			for(int i = 0; i < messages.Count; i++) {
				if(messages[i].ID == id) {
					result = messages[i];
				}
				if(result == null) {
					result = FindMessage(messages[i].Replies, id);
				}
				if(result != null) break;
			}
			return result;
		}

		/// <summary>
		/// Adds a new Message to a Page.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="username">The Username.</param>
		/// <param name="subject">The Subject.</param>
		/// <param name="dateTime">The Date/Time.</param>
		/// <param name="body">The Body.</param>
		/// <param name="parent">The Parent Message ID, or -1.</param>
		/// <returns>True if the Message has been added successfully.</returns>
		public bool AddMessage(PageInfo page, string username, string subject, DateTime dateTime, string body, int parent) {
			return page.Provider.AddMessage(page, username, subject, dateTime, body, parent);
		}

		/// <summary>
		/// Removes a Message.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="id">The ID of the Message to remove.</param>
		/// <param name="removeReplies">A value specifying whether or not to remove the replies.</param>
		/// <returns>True if the Message has been removed successfully.</returns>
		public bool RemoveMessage(PageInfo page, int id, bool removeReplies) {
			return page.Provider.RemoveMessage(page, id, removeReplies);
		}

		/// <summary>
		/// Modifies a Message.
		/// </summary>
		/// <param name="page">The Page.</param>
		/// <param name="id">The ID of the Message to modify.</param>
		/// <param name="username">The Username.</param>
		/// <param name="subject">The Subject.</param>
		/// <param name="dateTime">The Date/Time.</param>
		/// <param name="body">The Body.</param>
		/// <returns>True if the Message has been modified successfully.</returns>
		public bool ModifyMessage(PageInfo page, int id, string username, string subject, DateTime dateTime, string body) {
			return page.Provider.ModifyMessage(page, id, username, subject, dateTime, body);
		}

		#endregion

		/// <summary>
        /// Checks the validity of a Page name.
        /// </summary>
        /// <param name="name">The Page name.</param>
        /// <returns>True if the name is valid.</returns>
        public static bool IsValidName(string name) {
            if(name.Replace(" ", "").Length == 0 ||
                name.Contains("?") || name.Contains("<") || name.Contains(">") || name.Contains("|") || name.Contains(":") ||
                name.Contains("*") || name.Contains("\"") || name.Contains("/") || name.Contains("\\") || name.Contains("&") ||
				name.Contains("%") || name.Contains("'") || name.Contains("\"") || name.Contains("+") || name.EndsWith(".")) {
                return false;
            }
            else return true;
        }

    }

	/// <summary>
	/// Compares PageContent objects.
	/// </summary>
	public class PageContentDateComparer : IComparer<PageContent> {

		/// <summary>
		/// Compares two PageContent objects, using the DateTime as parameter.
		/// </summary>
		/// <param name="x">The first object.</param>
		/// <param name="y">The second object.</param>
		/// <returns>The result of the comparison (1, 0 or -1).</returns>
		public int Compare(PageContent x, PageContent y) {
			return x.LastModified.CompareTo(y.LastModified);
		}

	}

}
